_hook_dict = ({}, {})


class Hook(object):
    """Decorator for Hooks
    Use to decorate system hooks where plugins should be able to hook in
    """

    def __init__(self, function):
        self.function = function

    def __call__(self, *args, **kwargs):
        # Hooks before
        for function in _hook_dict[0].get(self.function.__name__, []):
            function(*args, **kwargs)
        # Main function
        ret = self.function(*args, **kwargs)
        # Hooks after
        for function in _hook_dict[1].get(self.function.__name__, []):
            function(*args, **kwargs)
        return ret


class HookBefore(object):
    """Decorator for functions to be called before a Hook-Function is called"""

    def __init__(self, name):
        self.name = name

    def __call__(self, function):
        _hook_dict[0].setdefault(self.name, []).append(function)
        return function


class HookAfter(object):
    """Decorator for functions to be called after a Hook-Function is called"""

    def __init__(self, name):
        self.name = name

    def __call__(self, function):
        _hook_dict[1].setdefault(self.name, []).append(function)
        return function
